#ifndef AIGroundNetworkLoader_h
#define AIGroundNetworkLoader_h

#include <Airport/Airport.h>
#include "AINetworkVertex.h"

#include <map>
#include <set>
#include <utility>
#include <xml/easyxml.hxx>

class AIGroundNetworkLoader: public XMLVisitor {
public:
        AIGroundNetworkLoader(Airport::SmartRef airport);
        virtual ~AIGroundNetworkLoader();
        void startXML();
        void endXML();
        void startElement(const char * name, const XMLAttributes &atts);
protected:
        void parseParking(const XMLAttributes& atts);
        void parseNode(const XMLAttributes& atts);
        void parseArc(const XMLAttributes& atts);
        static double processPosition(const string &pos);

        class Arc {
        public:
                unsigned int source;
                unsigned int target;
                bool isPushBackRoute;
                wxString name;
                
                Arc(unsigned int source, unsigned int target, bool isPushBackRoute, wxString name) {
                        this->source=source;
                        this->target=target;
                        this->isPushBackRoute=isPushBackRoute;
                        this->name=name;
                }
                
                bool operator<(const Arc& arc) const {
                        return target<arc.target;
                }
        };
        
        typedef std::set<Arc> ArcSet;
        typedef std::map<unsigned int,ArcSet> ArcSetMap;
        typedef std::map<unsigned int, AINetworkVertex::SmartRef> NodeIndexMap;
        
        Airport::SmartRef _airport;
        NodeIndexMap _nodeIndexMap;
        ArcSetMap _arcBySourceNodeMap;
};

#endif

