#ifndef AINetworkNode_h
#define AINetworkNode_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "AINetworkVertex.h"

class AINetworkNode: public AINetworkVertex {
        
        DECLARE_CLASS(AINetworkNode)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(AINetworkNode);
        
public:
        typedef enum {
                HoldPointType_None,
                HoldPointType_Normal,
                HoldPointType_CAT_II_III,
                HoldPointType_PushBack
        } HoldPointType;
        
	AINetworkNode(TransactionManager* transactionManager,
                      Point3D position,
                      bool isOnRunway=false,
                      HoldPointType holdPointType=HoldPointType_None);
        
        virtual ~AINetworkNode();
        
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        void PaintNode(wxDC& dc, MapEditor* editor, wxColour colour);
        
        virtual bool Hit(MapEditor* editor, TDRectangle r);
        
        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        TDRectangle GetGeographicBounds();
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(bool,IsOnRunway,isOnRunway)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(HoldPointType,HoldPointType,holdPointType)
        
        bool IsOnRunway() const {
                return GetIsOnRunway();
        }

        static double Radius;
protected:
        typedef struct {
                bool isOnRunway;
                HoldPointType holdPointType;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(AINetworkNode,AINetworkVertex,State)
};

#endif

