#include "AirportPropertyEditor.h"

IMPLEMENT_CLASS(AirportPropertyEditor,wxScrolledWindow);

#include "Airport.h"

#include <Transactions/TransactionManager.h>

enum {
        AirportPropertyEditor_ICAOID=wxID_HIGHEST,
        AirportPropertyEditor_Name,
        AirportPropertyEditor_Elevation,
        AirportPropertyEditor_HasControlTower,
        AirportPropertyEditor_HasDefaultBuildings,
        AirportPropertyEditor_FrequencyList,
        AirportPropertyEditor_FrequencyType,
        AirportPropertyEditor_Frequency,
        AirportPropertyEditor_FrequencyName,
        AirportPropertyEditor_AddFrequency,
        AirportPropertyEditor_UpdateFrequency,
        AirportPropertyEditor_RemoveFrequency
};

BEGIN_EVENT_TABLE(AirportPropertyEditor,wxScrolledWindow)
        EVT_TEXT_ENTER(AirportPropertyEditor_ICAOID,AirportPropertyEditor::OnTextEnter)
        EVT_TEXT_ENTER(AirportPropertyEditor_Name,AirportPropertyEditor::OnTextEnter)
        EVT_TEXT_ENTER(AirportPropertyEditor_Elevation,AirportPropertyEditor::OnTextEnter)
        EVT_CHECKBOX(AirportPropertyEditor_HasControlTower,AirportPropertyEditor::OnCheckbox)
        EVT_CHECKBOX(AirportPropertyEditor_HasDefaultBuildings,AirportPropertyEditor::OnCheckbox)
        EVT_LIST_ITEM_SELECTED(AirportPropertyEditor_FrequencyList,AirportPropertyEditor::OnFrequencySelected)
        EVT_LIST_ITEM_DESELECTED(AirportPropertyEditor_FrequencyList,AirportPropertyEditor::OnFrequencyDeselected)
        EVT_BUTTON(AirportPropertyEditor_AddFrequency,AirportPropertyEditor::OnAddFrequency)
        EVT_BUTTON(AirportPropertyEditor_UpdateFrequency,AirportPropertyEditor::OnUpdateFrequency)
        EVT_BUTTON(AirportPropertyEditor_RemoveFrequency,AirportPropertyEditor::OnRemoveFrequency)
END_EVENT_TABLE()

AirportPropertyEditor::AirportPropertyEditor():
        wxScrolledWindow()
{
}

AirportPropertyEditor::AirportPropertyEditor(wxWindow* parent,
                                Airport *airport,
                                wxWindowID id,
                                const wxPoint& pos,
                                const wxSize& size,
                                long style,
                                const wxString& name):
        wxScrolledWindow()
{
        Create(parent,airport,id,pos,size,style,name);
}

AirportPropertyEditor::~AirportPropertyEditor() {
        _airport->UnregisterChangeListener(this);
}

bool AirportPropertyEditor::Create(wxWindow* parent,
                                Airport *airport,
                                wxWindowID id,
                                const wxPoint& pos,
                                const wxSize& size,
                                long style,
                                const wxString& name)
{
        if (!wxScrolledWindow::Create(parent,id,pos,size,style,name))
                return false;
        
        
        
        _airport=airport;
        _airport->RegisterChangeListener(this);

        _frequencyTypeChoices.Add(wxT("AWOS"));
        _frequencyTypeChoices.Add(wxT("UNICOM"));
        _frequencyTypeChoices.Add(wxT("Clearance"));
        _frequencyTypeChoices.Add(wxT("Ground"));
        _frequencyTypeChoices.Add(wxT("Tower"));
        _frequencyTypeChoices.Add(wxT("Approach"));
        _frequencyTypeChoices.Add(wxT("Departure"));
        
        _icaoID=new wxTextCtrl(this,AirportPropertyEditor_ICAOID);
        _icaoID->SetWindowStyleFlag(wxTE_PROCESS_ENTER);
        _name=new wxTextCtrl(this,AirportPropertyEditor_Name);
        _name->SetWindowStyleFlag(wxTE_PROCESS_ENTER);
        _elevation=new wxTextCtrl(this,AirportPropertyEditor_Elevation);
        _elevation->SetWindowStyleFlag(wxTE_PROCESS_ENTER);
        _hasControlTower=new wxCheckBox(this,AirportPropertyEditor_HasControlTower,
                                        wxT("Has Control Tower"));
        _hasDefaultBuildings=new wxCheckBox(this,AirportPropertyEditor_HasDefaultBuildings,
                                        wxT("Has Default Buildings"));
        _frequencyList=new wxListCtrl(this,AirportPropertyEditor_FrequencyList);
        _frequencyList->SetWindowStyle(wxLC_REPORT|
                        wxLC_HRULES|
                        wxLC_VRULES|
                        wxLC_SINGLE_SEL);
        _frequencyType=new wxChoice(this,AirportPropertyEditor_FrequencyType);
        _frequency=new wxTextCtrl(this,AirportPropertyEditor_Frequency);
        _frequencyName=new wxTextCtrl(this,AirportPropertyEditor_FrequencyName);

        _addFrequency=new wxButton(this,AirportPropertyEditor_AddFrequency,wxT("Add"));
        _updateFrequency=new wxButton(this,AirportPropertyEditor_UpdateFrequency,wxT("Update"));
        _removeFrequency=new wxButton(this,AirportPropertyEditor_RemoveFrequency,wxT("Remove"));
        
        _updateFrequency->Disable();
        _removeFrequency->Disable();
        
        _frequencyList->InsertColumn(0,wxT("Name"));
        _frequencyList->InsertColumn(1,wxT("Type"));
        _frequencyList->InsertColumn(2,wxT("Frequency"));
        
        _frequencyType->Append(_frequencyTypeChoices);
        
        wxBoxSizer *topSizer=new wxBoxSizer(wxVERTICAL);
        wxFlexGridSizer *gridSizer;
        
        wxStaticBoxSizer *genBoxSizer=new wxStaticBoxSizer(wxVERTICAL,this,wxT("General"));
        
        gridSizer=new wxFlexGridSizer(2,5,5);
        gridSizer->AddGrowableCol(1,1);
        
        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("ICAO-ID")));
        gridSizer->Add(_icaoID,wxSizerFlags(0).Expand());
        
        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("Name")));
        gridSizer->Add(_name,wxSizerFlags(0).Expand());

        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("Elevation")));
        gridSizer->Add(_elevation,wxSizerFlags(0).Expand());
        
        genBoxSizer->Add(gridSizer,wxSizerFlags(0).Expand());
        
        genBoxSizer->Add(_hasControlTower,wxSizerFlags(0).Expand());

        genBoxSizer->Add(_hasDefaultBuildings,wxSizerFlags(0).Expand());
        
        topSizer->Add(genBoxSizer,wxSizerFlags(0).Expand());

        wxStaticBoxSizer *freqBoxSizer=new wxStaticBoxSizer(wxVERTICAL,this,wxT("Frequencies"));
        
        freqBoxSizer->Add(_frequencyList,wxSizerFlags(1).Expand());

        gridSizer=new wxFlexGridSizer(2,5,5);
        gridSizer->AddGrowableCol(1,1);
        
        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("Type")));
        gridSizer->Add(_frequencyType,wxSizerFlags(0).Expand());
        
        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("Frequency")));
        gridSizer->Add(_frequency,wxSizerFlags(0).Expand());
        
        gridSizer->Add(new wxStaticText(this,wxID_ANY,wxT("Name")));
        gridSizer->Add(_frequencyName,wxSizerFlags(0).Expand());
        
        wxBoxSizer *buttonsSizer=new wxBoxSizer(wxVERTICAL);
        buttonsSizer->Add(_addFrequency,wxSizerFlags(0).Border(wxVERTICAL,5));
        buttonsSizer->Add(_updateFrequency,wxSizerFlags(0).Border(wxVERTICAL,5));
        buttonsSizer->Add(_removeFrequency,wxSizerFlags(0).Border(wxVERTICAL,5));
        
        wxBoxSizer *freqInputsSizer=new wxBoxSizer(wxHORIZONTAL);
        
        freqInputsSizer->Add(gridSizer,wxSizerFlags(1).Expand());
        freqInputsSizer->Add(buttonsSizer,wxSizerFlags().Align(wxALIGN_TOP));
        
        freqBoxSizer->Add(freqInputsSizer,wxSizerFlags(0).Expand());
        
        topSizer->Add(freqBoxSizer,wxSizerFlags(1).Expand());
        
        SetSizer(topSizer);
        
        topSizer->SetSizeHints(this);
        
        SetScrollRate(10,10);
        
        Fit();
        
        UpdateFields();
        
        return true;
}

void AirportPropertyEditor::OnTextEnter(wxCommandEvent& evt) {
        TransactionManager* transmgr=_airport->GetTransactionManager();
        switch (evt.GetId()) {
        case AirportPropertyEditor_ICAOID:
                _airport->SetICAOID(_icaoID->GetValue());
                transmgr->Commit(wxT("Changing Airport ICAO ID"));
                break;
        case AirportPropertyEditor_Name:
                _airport->SetName(_name->GetValue());
                transmgr->Commit(wxT("Changing Airport Name"));
                break;
        case AirportPropertyEditor_Elevation:
                double newElev;
                wxString elevStr=_elevation->GetValue();
                if (!elevStr.ToDouble(&newElev)) {
                        ::wxMessageBox(wxT("Elevation must be a number"),
                                       wxT("Invalid Input"),
                                       wxOK|wxICON_WARNING,
                                       this);
                        break;
                }
                _airport->SetElevation(newElev);
                transmgr->Commit(wxT("Changing Airport Elevation"));
                break;
        }
}

void AirportPropertyEditor::OnCheckbox(wxCommandEvent& evt) {
        TransactionManager* transmgr=_airport->GetTransactionManager();
        switch (evt.GetId()) {
        case AirportPropertyEditor_HasControlTower:
                _airport->SetControlTower(_hasControlTower->GetValue());
                transmgr->Commit(wxT("Changing Control Tower Flag"));
                break;
        case AirportPropertyEditor_HasDefaultBuildings:
                _airport->SetDefaultBuildings(_hasDefaultBuildings->GetValue());
                transmgr->Commit(wxT("Changing Default Buildings Flag"));
                break;
        }
}

void AirportPropertyEditor::OnFrequencySelected(wxListEvent& evt) {
        size_t idx=evt.GetIndex();
        const FrequencyEntryArray& freqs=_airport->GetFrequencies();
        const FrequencyEntry& entry=freqs.Item(idx);
        
        size_t freqTypeIdx;
        freqTypeIdx=entry.GetType()-FrequencyEntry::FrequencyType_AWOS;
        
        _frequencyType->SetSelection(freqTypeIdx);
        _frequency->SetValue(entry.GetFrequency());
        _frequencyName->SetValue(entry.GetName());
        
        _updateFrequency->Enable();
        _removeFrequency->Enable();
}

void AirportPropertyEditor::OnFrequencyDeselected(wxListEvent& evt) {
        _frequency->Clear();
        _frequencyName->Clear();
        
        _updateFrequency->Disable();
        _removeFrequency->Disable();
}

void AirportPropertyEditor::OnAddFrequency(wxCommandEvent& evt) {
        TransactionManager* transmgr=_airport->GetTransactionManager();
        FrequencyEntry::FrequencyType type;
        
        type=(FrequencyEntry::FrequencyType)(_frequencyType->GetSelection()+FrequencyEntry::FrequencyType_AWOS);
        
        FrequencyEntryArray entries=_airport->GetFrequencies();
        wxString name=_frequencyName->GetValue();
        entries.Add(FrequencyEntry(type,_frequency->GetValue(),_frequencyName->GetValue()));

        _airport->SetFrequencies(entries);
        
        transmgr->Commit(wxT("Adding frequency ")+name);
}

void AirportPropertyEditor::OnUpdateFrequency(wxCommandEvent& evt) {
        TransactionManager* transmgr=_airport->GetTransactionManager();
        FrequencyEntry::FrequencyType type;
        
        type=(FrequencyEntry::FrequencyType)(_frequencyType->GetSelection()+FrequencyEntry::FrequencyType_AWOS);
        
        long item=_frequencyList->GetNextItem(-1,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
        wxASSERT(item!=-1);
        
        FrequencyEntryArray entries=_airport->GetFrequencies();
        wxString name=_frequencyName->GetValue();
        FrequencyEntry &entry=entries.Item(item);
        entry.SetType(type);
        entry.SetFrequency(_frequency->GetValue());
        entry.SetName(_frequencyName->GetValue());
        
        _airport->SetFrequencies(entries);
        
        transmgr->Commit(wxT("Changing frequency ")+name);
}

void AirportPropertyEditor::OnRemoveFrequency(wxCommandEvent& evt) {
        TransactionManager* transmgr=_airport->GetTransactionManager();

        long item=_frequencyList->GetNextItem(-1,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
        wxASSERT(item!=-1);
        
        FrequencyEntryArray entries=_airport->GetFrequencies();
        FrequencyEntry entry=entries.Item(item);
        entries.RemoveAt(item);
        
        _airport->SetFrequencies(entries);
        
        transmgr->Commit(wxT("Removing frequency ")+entry.GetName());
}

void AirportPropertyEditor::BeforeObjectChange(TransactionObject* object) {
        // NO-OP
}

void AirportPropertyEditor::AfterObjectChange(TransactionObject* object) {
        UpdateFields();
}

void AirportPropertyEditor::UpdateFields() {
        _icaoID->SetValue(_airport->GetICAOID());
        _name->SetValue(_airport->GetName());
        
        wxString elevStr;
        elevStr.Printf(wxT("%.1f"),_airport->GetElevation());
        _elevation->SetValue(elevStr);
        _hasControlTower->SetValue(_airport->HasControlTower());
        _hasDefaultBuildings->SetValue(_airport->HasDefaultBuildings());
        
        _frequencyList->Freeze();
        long item=_frequencyList->GetNextItem(-1,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
        _frequencyList->DeleteAllItems();
        const FrequencyEntryArray& freqs=_airport->GetFrequencies();
        
        for (size_t i=0;i<freqs.GetCount();i++) {
                long idx=_frequencyList->InsertItem(i,freqs.Item(i).GetName());
                
                size_t freqTypeIdx;
                freqTypeIdx=freqs.Item(i).GetType()-FrequencyEntry::FrequencyType_AWOS;
                
                _frequencyList->SetItem(idx,1,_frequencyTypeChoices.Item(freqTypeIdx));
                _frequencyList->SetItem(idx,2,freqs.Item(i).GetFrequency());
        }
        
        if (item>=0 && (size_t)item>=freqs.GetCount())
                item=freqs.GetCount()-1;
        
        if (item!=-1) {
                _frequencyList->SetItemState(item,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                _frequencyList->EnsureVisible(item);
        }
        
        _frequencyList->SetColumnWidth(0,wxLIST_AUTOSIZE);
        _frequencyList->SetColumnWidth(1,wxLIST_AUTOSIZE);
        _frequencyList->SetColumnWidth(2,wxLIST_AUTOSIZE_USEHEADER);
        
        _frequencyList->Thaw();
}
