#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "PavementSelectionEndHandle.h"
#include "PavementSelection.h"

#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

#include <Math/sg_geodesy.h>

#include <Map/MapProjection.h>

IMPLEMENT_CLASS(PavementSelectionEndHandle,MapSelectionHandle);

PavementSelectionEndHandle::PavementSelectionEndHandle(MapSelection* selection, unsigned int end):
        MapSelectionHandle(selection),
        _end(end)
{
}

PavementSelectionEndHandle::~PavementSelectionEndHandle()
{
}

void PavementSelectionEndHandle::StartDrag(Point3D startPosition) {
        PavementSelection* selection=(PavementSelection*)GetSelection();
        PavementMapObject* object=selection->GetContent();
        MapEditor* editor=selection->GetSelectionManager()->GetEditor();
        MapProjection* projection=editor->GetProjection();
        
        const Point3D* endpts=object->GetEndCenters();
        Point3D localendpts[2];
        for (int i=0;i<2;i++)
                localendpts[i]=projection->ConvertToLocal(endpts[i]);
        _handleDelta=startPosition-localendpts[_end];
}

/*
 * Drag this handle to the given position in the local coordinate system
 * of the editor associated with our selection manager */
void PavementSelectionEndHandle::Drag(Point3D currentPosition)
{
        PavementSelection* selection=(PavementSelection*)GetSelection();
        PavementMapObject* object=selection->GetContent();
        MapEditor* editor=selection->GetSelectionManager()->GetEditor();
        MapProjection* projection=editor->GetProjection();
        
        const Point3D* endpts=object->GetEndCenters();
        Point3D localendpts[2];
        for (int i=0;i<2;i++)
                localendpts[i]=projection->ConvertToLocal(endpts[i]);
        
        localendpts[_end]=currentPosition-_handleDelta;
        
        Point3D newendpts[2];
        for (int i=0;i<2;i++)
                newendpts[i]=projection->ConvertFromLocal(localendpts[i]);
        
        /*
         * Determine new length and heading
         */
        double alt=object->GetAirport()->GetElevation()*0.3048;
        double az1, az2, length;
        geo_inverse_wgs_84( alt,
                            newendpts[0].lat(), newendpts[0].lon(),
                            newendpts[1].lat(), newendpts[1].lon(),
                            &az1, &az2, &length );
        length/=0.3048;

        /*
         * Determine new center
         *
         * Here we assume that the user wants only the dragged end
         * to move _in the current projection_. This solution
         * emulates manually moving the center by half the way after
         * changing the length. It also seems to work quite well.
         */
        Point3D localCenter=(localendpts[0]+localendpts[1])/2.0;
        Point3D center=projection->ConvertFromLocal(localCenter);

        object->SetCenter(center);
        object->SetLength(length);
        object->SetHeading(az1);
}

