#include "MapCalibratedBackgroundImageSelection.h"
#include "MapCalibratedBackgroundImage.h"
#include "MapCalibratedBackgroundImageActionsHandler.h"
#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(MapCalibratedBackgroundImageSelection,MapSimpleBackgroundImageSelection);

MapCalibratedBackgroundImageSelection::MapCalibratedBackgroundImageSelection(MapSelectionManager* manager, MapCalibratedBackgroundImage* content):
        MapSimpleBackgroundImageSelection(manager,content) {
}

MapCalibratedBackgroundImageSelection::~MapCalibratedBackgroundImageSelection() {
}

void MapCalibratedBackgroundImageSelection::ProvideActionMenuForClass(wxMenu& actionMenu, wxClassInfo* targetClass) {
        MapSimpleBackgroundImageSelection::ProvideActionMenuForClass(actionMenu,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(MapCalibratedBackgroundImageSelection)))
                return;
        bool isSingleton=(GetSelectionManager()->GetSelections().GetCount()==1);
        wxMenu* calibrationMenu=new wxMenu();
        calibrationMenu->SetEventHandler(actionMenu.GetEventHandler());
        calibrationMenu->SetClientData(actionMenu.GetClientData());
        
        wxMenuItem* item;
        
        item=calibrationMenu->Append(wxID_ANY,wxT("Fixed Scale, No Rotation"));
        calibrationMenu->GetEventHandler()->Connect(item->GetId(),
                                                    wxEVT_COMMAND_MENU_SELECTED,
                                                    wxCommandEventHandler(MapCalibratedBackgroundImageActionsHandler::OnFixedScaleNoRotate),
                                                    NULL,
                                                    MapCalibratedBackgroundImageActionsHandler::GetInstance());
        item=calibrationMenu->Append(wxID_ANY,wxT("Fixed Scale, Rotation"));
        calibrationMenu->GetEventHandler()->Connect(item->GetId(),
                                                    wxEVT_COMMAND_MENU_SELECTED,
                                                    wxCommandEventHandler(MapCalibratedBackgroundImageActionsHandler::OnFixedScaleRotate),
                                                    NULL,
                                                    MapCalibratedBackgroundImageActionsHandler::GetInstance());
        item=calibrationMenu->Append(wxID_ANY,wxT("Uniform Scale, No Rotation"));
        calibrationMenu->GetEventHandler()->Connect(item->GetId(),
                                                    wxEVT_COMMAND_MENU_SELECTED,
                                                    wxCommandEventHandler(MapCalibratedBackgroundImageActionsHandler::OnUniformScaleNoRotate),
                                                    NULL,
                                                    MapCalibratedBackgroundImageActionsHandler::GetInstance());
        item=calibrationMenu->Append(wxID_ANY,wxT("Uniform Scale, Rotation"));
        calibrationMenu->GetEventHandler()->Connect(item->GetId(),
                                                    wxEVT_COMMAND_MENU_SELECTED,
                                                    wxCommandEventHandler(MapCalibratedBackgroundImageActionsHandler::OnUniformScaleRotate),
                                                    NULL,
                                                    MapCalibratedBackgroundImageActionsHandler::GetInstance());
        
        actionMenu.Append(wxID_HIGHEST,wxT("Calibration"),calibrationMenu);
}

void MapCalibratedBackgroundImageSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSimpleBackgroundImageSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(MapCalibratedBackgroundImageSelection)))
                return;
        
        propgrid->Append(wxFloatProperty(wxT("Angle"), wxT("MapCalibratedBackgroundImageSelection_Angle")));
}

void MapCalibratedBackgroundImageSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSimpleBackgroundImageSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(MapCalibratedBackgroundImageSelection)))
                return;

        propgrid->SetPropertyValue(wxT("MapCalibratedBackgroundImageSelection_Angle"),GetContent()->GetAngle()*180.0/M_PI);
}

void MapCalibratedBackgroundImageSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        MapSimpleBackgroundImageSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(MapCalibratedBackgroundImageSelection)))
                return;
        MapCalibratedBackgroundImage* firstContent=wxDynamicCast(firstSelection,MapCalibratedBackgroundImageSelection)->GetContent();
        MapCalibratedBackgroundImage* myContent=GetContent();

        if (firstContent->GetAngle()!=myContent->GetAngle())
                propgrid->SetPropertyUnspecified(wxT("MapCalibratedBackgroundImageSelection_Angle"));
}

void MapCalibratedBackgroundImageSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("MapCalibratedBackgroundImageSelection_Angle"))==0) {
                GetContent()->SetAngle(value.GetDouble()*M_PI/180.0);
        } else {
                MapSimpleBackgroundImageSelection::UpdateObjectFromProperty(property);
        }
}

