#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "TransactionObject.h"
#include <wx/listimpl.cpp>

WX_DEFINE_LIST(TransactionObjectChangeListenerList);

IMPLEMENT_ABSTRACT_CLASS(TransactionObject,wxObject);

TransactionObject::TransactionObject(TransactionManager* manager):
                _refCount(0),
                _manager(manager),
                _dirty(false)
{
	_manager->Hold();
}

TransactionObject::~TransactionObject() {
        wxASSERT(_manager!=NULL);
        wxASSERT_MSG(_refCount==0,
                wxT("Trying to destroy referenced transaction object")); 
        _manager->DeleteCopy(this);
	_manager->Unhold();
}

void TransactionObject::TriggerBeforeObjectChange() {
        // trigger object change listeners
        TransactionObjectChangeListenerList::Node* node;
        for (node=_changeListeners.GetFirst();node;node=node->GetNext()) {
                TransactionObjectChangeListener* listener;
                listener=node->GetData();
                listener->BeforeObjectChange(this);
        }
}

void TransactionObject::TriggerAfterObjectChange() {
        // trigger object change listeners
        TransactionObjectChangeListenerList::Node* node;
        for (node=_changeListeners.GetFirst();node;node=node->GetNext()) {
                TransactionObjectChangeListener* listener;
                listener=node->GetData();
                listener->AfterObjectChange(this);
        }
}

void TransactionObject::BeforeWrite() {
        TriggerBeforeObjectChange();
        // ensure storage of memento
        if (!_dirty) {
                _dirty=true;
                wxASSERT(_manager);
                _manager->StoreCopy(this);
        }
}

void TransactionObject::AfterWrite() {
        TriggerAfterObjectChange();
}

