//*** Menu Tool Bar

//

// This header file holds the declarations for both the wxMenuToolBar

// and the wxMenuButton.

//



#ifndef MENUTOOLBAR_H

#define MENUTOOLBAR_H



#include "wx/frame.h"



DECLARE_EVENT_TYPE(wxEVT_FILL_MENU, -1)



// *

// * wxMenuButton - Can be used wherever you can use a standard wxBitmapButton

// *

// * Implements a small pull-down triangle (v), which, when clicked, will display

// * a pop-up menu. The menu can be filled by responding to the wxEVT_FILL_MENU event,

// * as in:

//

//    EVT_COMMAND(MY_BUTTON_ID, wxEVT_FILL_MENU, MyClass::MyFillRoutine)

//

// The MyClass::MyFillRoutine() should look something like this:

//

// void MyClass::MyFillRoutine(wxCommandEvent& event) {

//   wxMenu *menu = (wxMenu *)event.GetClientData();

//   if(menu->GetMenuItemCount() <= 0) {

//     menu->Append(MY_ID1, wxT("My Menu Item 1"));

//     menu->AppendSeparator();

//     menu->Append(MY_ID2, wxT("My Menu Item 2"));

//     // etc.

//   }

// }





class wxMenuButton : public wxBitmapButton

{

public:

  wxMenuButton(wxToolBar *toolBar, int ID)  { Create(toolBar, toolBar, ID); }

  wxMenuButton(wxWindow *window, int ID)   { Create(window, NULL, ID); }

  ~wxMenuButton();



  void Create(wxToolBar *toolBar, int ID) { Create(toolBar, toolBar, ID); }

  void Create(wxWindow *window, int ID)  { Create(window, NULL, ID); }



  void DoProcessLeftClick(wxMouseEvent& event);



  wxMenu *m_menu;



  void FillMenu();



private:

  void Create(wxWindow *window, wxToolBar *toolBar, int ID);

  wxToolBar *m_toolBar;

};





// wxMenuTool - is only used internal to the implementation of wxMenuToolBar.

//

// You should never have to use it yourself



class wxMenuTool {

public:

  wxMenuTool(wxToolBarToolBase *new_tool, int toolId);

  ~wxMenuTool();



  wxToolBarToolBase *m_tool;

  int m_toolId;

  wxMenuButton *m_button;

  wxMenu *m_menu;

};



WX_DECLARE_LIST(wxMenuTool, wxMenuToolList);





// *** wxMenuToolBar  -  A replacement for wxToolBar which implements menu buttons

//                       and pull-down buttons

//

// A menu button is a standard looking toolbar tool which, when clicked, pops up a 

// menu which can be selected.

//

// A pull-down button presents a small black triangle which, when clicked, pops up

// a menu which can be selected. These buttons are typically used for a list of previous

// actions (for example, previous web pages visited).

//

// All menus can be filled by responding to the wxEVT_FILL_MENU event, as in:

//

//    EVT_COMMAND(MY_BUTTON_ID, wxEVT_FILL_MENU, MyClass::MyFillRoutine)

//

// The MyClass::MyFillRoutine() should look something like this:

//

// void MyClass::MyFillRoutine(wxCommandEvent& event) {

//   wxMenu *menu = (wxMenu *)event.GetClientData();

//   if(menu->GetMenuItemCount() <= 0) {

//     menu->Append(MY_ID1, wxT("My Menu Item 1"));

//     menu->AppendSeparator();

//     menu->Append(MY_ID2, wxT("My Menu Item 2"));

//     // etc.

//   }

// }



class wxMenuToolBar : public wxToolBar

{

public:

  wxMenuToolBar();

  wxMenuToolBar(wxFrame* parent, 

          wxWindowID id, 

          const wxPoint& pos = wxDefaultPosition, 

          const wxSize& size = wxDefaultSize, 

          long style = wxTB_HORIZONTAL | wxNO_BORDER, 

          const wxString& name = wxPanelNameStr);

  ~wxMenuToolBar();



  // NOTE:  label, shortHelpString, are not implemented on Macintosh (they work on MSW)

  void AddMenuButtonTool(int toolId, 

                         const wxString& label, 

                         const wxBitmap& bitmap1, 

                         const wxString& shortHelpString = wxT(""));



  // NOTE:  label, shortHelpString, are not implemented on all platforms and are only 

  //        included for possible future upgrades

  wxMenuButton *AddMenuPulldownTool(int toolId, 

                         const wxString& label, 

                         const wxString& shortHelpString = wxT(""));



  void NewMenuTool(wxMenuTool *menu_tool);



  void DoProcessLeftClick(wxMouseEvent& event);



  wxFrame *m_frame;

  wxMenuToolList *m_menuTools;

};





#ifdef __WXMAC__

// On Macintosh, the toolbar does not transmit wxEVT_LEFT_DOWN events, so we need to

// replace MenuButton Tools with bitmap buttons

class wxMacMenuTool : public wxBitmapButton

{

  public:

    wxMacMenuTool(wxMenuToolBar* parent, wxWindowID id, const wxBitmap &bmp, const wxPoint& pos = wxDefaultPosition, 

      const wxSize& size = wxDefaultSize, long style = wxTB_HORIZONTAL | wxNO_BORDER);

    ~wxMacMenuTool();



    void DoProcessLeftClick(wxMouseEvent& event);

    wxMenu *m_menu;

    wxMenuToolBar *m_toolBar;

};



#endif



#endif

